#include "delay.h"
#include "esp8266.h"
/** @addtogroup TIM_TimeBase
 * @{
 */

static TIM_TimeBaseInitType TIM_TimeBaseStructure;
static OCInitType TIM_OCInitStructure;
uint16_t PrescalerValue = 0;

static void NVIC_Configuration(void);
static void TIM6_Configuration(void);
static void TIM7_Configuration(void);

uint8_t TIM6_IT_FLAG = 0;

void delay(vu32 nCount)
{
    vu32 index = 0;
    for (index = (34000 * nCount); index != 0; index--)
    {
    }
}

// void TIM6_IRQHandler(void)
// {
//     if( TIM_GetFlagStatus(TIM6, TIM_INT_UPDATE) )
//     {
//         TIM6_IT_FLAG = 1;
//     }
//     TIM_ClearFlag(TIM6,TIM_INT_UPDATE); 
// }

/**
 * @brief  Main program
 */
void TIM_init(void)
{
    //不使用中断的方式延时
    // /* NVIC Configuration */
    // NVIC_Configuration();

    /* TIM Configuration */
    TIM6_Configuration();
    TIM7_Configuration();
}
/**
 * @brief 毫秒级延时
 * 
 * @param ms 
 */
void delay_ms( uint16_t ms )
{
    TIM7->AR = ms;
    TIM_Enable( TIM7, ENABLE );
    while ( !TIM_GetFlagStatus(TIM7, TIM_INT_UPDATE) );
    TIM_ClearFlag(TIM7,TIM_INT_UPDATE); 
    TIM_Enable( TIM7, DISABLE );
}
/**
 * @brief 微秒级延时
 * 
 * @param us 
 */
void delay_us( uint16_t us )
{
    TIM6->AR = us;
    TIM_Enable( TIM6, ENABLE );
    while ( !TIM_GetFlagStatus(TIM6, TIM_INT_UPDATE) );
    TIM_ClearFlag(TIM6,TIM_INT_UPDATE); 
    TIM_Enable( TIM6, DISABLE );
}
/**
 * @brief  Configures tim6 clocks.
 */
static void TIM6_Configuration(void)
{
    /* Compute the prescaler value */
    PrescalerValue = (64-1); //(uint16_t) (SystemCoreClock / 12000000) - 1;
    /* Time base configuration */
    TIM_TimeBaseStructure.Period    = 999;
    TIM_TimeBaseStructure.Prescaler = (64-1);
    TIM_TimeBaseStructure.ClkDiv    = 0;
    TIM_TimeBaseStructure.CntMode   = TIM_CNT_MODE_UP;

    TIM_InitTimeBase(TIM6, &TIM_TimeBaseStructure);

    /* Prescaler configuration */
    TIM_ConfigPrescaler(TIM6, PrescalerValue, TIM_PSC_RELOAD_MODE_IMMEDIATE);

    /* TIM6 enable update irq */
    TIM_ConfigInt(TIM6, TIM_INT_UPDATE, ENABLE);

    /* TIM6 enable counter */
    TIM_Enable(TIM6, DISABLE);
}

static void TIM7_Configuration(void)
{
    /* Compute the prescaler value */
    PrescalerValue = (64000-1); //(uint16_t) (SystemCoreClock / 12000000) - 1;
    /* Time base configuration */
    TIM_TimeBaseStructure.Period    = 999;
    TIM_TimeBaseStructure.Prescaler = (64000-1);
    TIM_TimeBaseStructure.ClkDiv    = 0;
    TIM_TimeBaseStructure.CntMode   = TIM_CNT_MODE_UP;

    TIM_InitTimeBase(TIM7, &TIM_TimeBaseStructure);

    /* Prescaler configuration */
    TIM_ConfigPrescaler(TIM7, PrescalerValue, TIM_PSC_RELOAD_MODE_IMMEDIATE);

    /* TIM6 enable update irq */
    TIM_ConfigInt(TIM7, TIM_INT_UPDATE, ENABLE);

    /* TIM6 enable counter */
    TIM_Enable(TIM7, DISABLE);
}

/**
 * @brief  Configure the nested vectored interrupt controller.
 */
// static void NVIC_Configuration(void)
// {
//     NVIC_InitType NVIC_InitStructure;

//     /* Enable the TIM2 global Interrupt */
//     NVIC_InitStructure.NVIC_IRQChannel                   = TIM6_IRQn;
//     NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 8;
//     NVIC_InitStructure.NVIC_IRQChannelSubPriority        = 7;
//     NVIC_InitStructure.NVIC_IRQChannelCmd                = ENABLE;

//     NVIC_Init(&NVIC_InitStructure);
// }

#ifdef USE_FULL_ASSERT

/**
 * @brief  Reports the name of the source file and the source line number
 *         where the assert_param error has occurred.
 * @param file pointer to the source file name
 * @param line assert_param error line source number
 */
void assert_failed(const uint8_t* expr, const uint8_t* file, uint32_t line)
{
    /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

    while (1)
    {
    }
}

#endif

/**
 * @}
 */

/**
 * @}
 */