#include "env_i2c.h"
#include "delay.h"
#include "draw_api.h"
#include "esp8266.h"

void I2C2_Configuration(void)
{
	I2C_InitType  I2C_InitStructure;
	GPIO_InitType  GPIO_InitStructure; 

	RCC_EnableAPB1PeriphClk(RCC_APB1_PERIPH_I2C2,ENABLE);
	RCC_EnableAPB2PeriphClk(RCC_APB2_PERIPH_GPIOA,ENABLE);
	RCC_EnableAPB2PeriphClk(RCC_APB2_PERIPH_AFIO, ENABLE);

	GPIO_InitStructure.Pin =  I2C2_SCL_Pin_X | I2C2_SDA_Pin_X;
	GPIO_InitStructure.GPIO_Slew_Rate = GPIO_Slew_Rate_Low;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;//I2C必须开漏输出
	GPIO_InitStructure.GPIO_Alternate = GPIO_AF5_I2C2;
	GPIO_InitStructure.GPIO_Pull = GPIO_Pull_Up;
	GPIO_InitPeripheral(I2C2_GPIOX, &GPIO_InitStructure);	

	I2C_DeInit(I2C2);
	I2C_InitStructure.BusMode = I2C_BUSMODE_I2C;
	I2C_InitStructure.FmDutyCycle = I2C_FMDUTYCYCLE_2;
	I2C_InitStructure.OwnAddr1 = 0x30;//主机的I2C地址,随便写
	I2C_InitStructure.AckEnable = I2C_ACKDIS;
	I2C_InitStructure.AddrMode = I2C_ADDR_MODE_7BIT;
	I2C_InitStructure.ClkSpeed = 400000;

	I2C_Init(I2C2, &I2C_InitStructure);
	I2C_Enable(I2C2, ENABLE);
	
	delay_us(20);
}

void SHT30_Write(u8 msb, u8 lsb)
{
 	while(I2C_GetFlag(I2C2, I2C_FLAG_BUSY));
	
	I2C_GenerateSTART(I2C2, ENABLE);//开启I2C2
	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_MODE_FLAG));/*EV5,主模式*/
    
	I2C_Send7bitAddress(I2C2, I2C2_Slave_1_ADDR, I2C_DIRECTION_SEND);//器件地址 -- 默认0x44
	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_TXMODE_FLAG));

	I2C_SendData(I2C2, msb);
	while (!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_DATA_SENDED));

	I2C_SendData(I2C2, lsb);
	while (!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_DATA_SENDED));
	
	I2C_GenerateStop(I2C2, ENABLE);//关闭I2C2总线
}

int SHT30_ReadTempAndHumi(u8 *data) {
	int i;
	
	I2C_ConfigAck(I2C2, ENABLE);
	
	I2C_GenerateSTART(I2C2, ENABLE);
	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_MODE_FLAG));
	
	I2C_Send7bitAddress(I2C2, I2C2_Slave_1_ADDR, I2C_DIRECTION_SEND);
	i = 0;
	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_TXMODE_FLAG)) {
		if (++i > 200) {
			return -1;
		}
	}
	
	I2C_SendData(I2C2, 0xE0);
	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_DATA_SENDED));

	I2C_SendData(I2C2, 0x00);
	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_DATA_SENDED));
	
	I2C_GenerateSTART(I2C2, ENABLE);
	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_MODE_FLAG));
	
	I2C_Send7bitAddress(I2C2, I2C2_Slave_1_ADDR, I2C_DIRECTION_RECV);
	i = 0;
	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_RXMODE_FLAG)) {
		if (++i > 200) {
			return -1;
		}
	}
	
	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_DATA_RECVD_FLAG));
	data[0] = I2C_RecvData(I2C2);
	
	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_DATA_RECVD_FLAG));
	data[1] = I2C_RecvData(I2C2);

	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_DATA_RECVD_FLAG));
	data[2] = I2C_RecvData(I2C2);

	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_DATA_RECVD_FLAG));
	data[3] = I2C_RecvData(I2C2);
	
	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_DATA_RECVD_FLAG));
	data[4] = I2C_RecvData(I2C2);

	I2C_ConfigAck(I2C2, DISABLE);

	while(!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_DATA_RECVD_FLAG));
	data[5] = I2C_RecvData(I2C2);

	I2C_GenerateStop(I2C2, ENABLE);	
	I2C_ConfigAck(I2C2, ENABLE);
	
	return 0;
}

int i2c_master_recv(uint8_t* data, int len)
{
    uint8_t* recvBufferPtr = data;
    
#ifdef NON_REENTRANT
    if (Mutex_Flag)
        return -1;
    else
        Mutex_Flag = 1;
#endif
    
    while (I2C_GetFlag(I2C2, I2C_FLAG_BUSY));
    I2C_ConfigAck(I2C2, ENABLE);
    
    // send start
    I2C_GenerateStart(I2C2, ENABLE);
    while (!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_MODE_FLAG)) // EV5
    {
    }
    
    // send addr
    I2C_SendAddr7bit(I2C2, I2C2_Slave_1_ADDR, I2C_DIRECTION_RECV);

    while (!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_RXMODE_FLAG)) // EV6
    {
    }
    GPIO_WriteBit( GPIOA, GPIO_PIN_8, Bit_SET );
    // recv data
    if (len == 1)
    {
        I2C_ConfigAck(I2C2, DISABLE);
        (void)(I2C2->STS1); /// clear ADDR
        (void)(I2C2->STS2);

        I2C_GenerateStop(I2C2, ENABLE);
        
        while (!I2C_GetFlag(I2C2, I2C_FLAG_RXDATNE))
        {
        }
        *recvBufferPtr++ = I2C_RecvData(I2C2);
        len--;
    }
    else if (len == 2)
    {
        I2C2->CTRL1 |= 0x0800; /// set ACKPOS
        (void)(I2C2->STS1);
        (void)(I2C2->STS2);
        I2C_ConfigAck(I2C2, DISABLE);
        
        while (!I2C_GetFlag(I2C2, I2C_FLAG_BYTEF))
        {
        }
        
        I2C_GenerateStop(I2C2, ENABLE);

        *recvBufferPtr++ = I2C_RecvData(I2C2);
        len--;
        *recvBufferPtr++ = I2C_RecvData(I2C2);
        len--;
    }
    else
    {
        I2C_ConfigAck(I2C2, ENABLE);
        (void)(I2C2->STS1);
        (void)(I2C2->STS2);
        
        while (len)
        {
            if (len == 3)
            {
                while (!I2C_GetFlag(I2C2, I2C_FLAG_BYTEF))
                {
                }
                I2C_ConfigAck(I2C2, DISABLE);
                *recvBufferPtr++ = I2C_RecvData(I2C2);
                len--;
                while (!I2C_GetFlag(I2C2, I2C_FLAG_BYTEF))
                {
                }
                
                    I2C_GenerateStop(I2C2, ENABLE);
        
                *recvBufferPtr++ = I2C_RecvData(I2C2);
                len--;
                *recvBufferPtr++ = I2C_RecvData(I2C2);
                len--;
                
                break;
            }
            
            while (!I2C_CheckEvent(I2C2, I2C_EVT_MASTER_DATA_RECVD_FLAG)) // EV7
            {
            }
            *recvBufferPtr++ = I2C_RecvData(I2C2);
            len--;
        }
    }
    
    while (I2C_GetFlag(I2C2, I2C_FLAG_BUSY))
    {
    }
    
#ifdef NON_REENTRANT
    if (Mutex_Flag)
        Mutex_Flag = 0;
    else
        return -2;
#endif

    return 0;
}

#define CRC8_POLYNOMIAL 0x31

uint8_t CheckCrc8(uint8_t* const message, uint8_t initial_value)
{
    uint8_t  remainder;	    //余数
    uint8_t  i = 0, j = 0;  //循环变量

    /* 初始化 */
    remainder = initial_value;

    for(j = 0; j < 2;j++)
    {
        remainder ^= message[j];

        /* 从最高位开始依次计算  */
        for (i = 0; i < 8; i++)
        {
            if (remainder & 0x80)
            {
                remainder = (remainder << 1)^CRC8_POLYNOMIAL;
            }
            else
            {
                remainder = (remainder << 1);
            }
        }
    }

    /* 返回计算的CRC码 */
    return remainder;
}

/**
 * @brief	将SHT30接收的6个字节数据进行CRC校验，并转换为温度值和湿度值
 * @param	dat  —— 存储接收数据的地址（6个字节数组）
 * @retval	校验成功  —— 返回0
 * 			校验失败  —— 返回1，并设置温度值和湿度值为0
*/
uint8_t SHT30_Dat_To_Float(uint8_t* const dat, uint8_t* temperature, uint8_t* humidity)
{
	uint16_t recv_temperature = 0;
	uint16_t recv_humidity = 0;
	
	/* 校验温度数据和湿度数据是否接收正确 */
	if(CheckCrc8(dat, 0xFF) != dat[2] || CheckCrc8(&dat[3], 0xFF) != dat[5])
		return 1;
	
	/* 转换温度数据 */
	recv_temperature = ((uint16_t)dat[0]<<8)|dat[1];
	*temperature = -45 + 175*((float)recv_temperature/65535);
	
	/* 转换湿度数据 */
	recv_humidity = ((uint16_t)dat[3]<<8)|dat[4];
	*humidity = 100 * ((float)recv_humidity / 65535);
	
	return 0;
}


/*
-----------------------------------
©著作权归作者所有：来自51CTO博客作者MCUlover666的原创作品，请联系作者获取转载授权，否则将追究法律责任
【STM32Cube_15】使用硬件I2C读取温湿度传感器数据（SHT30）
https://blog.51cto.com/u_13640625/3032434
*/
