#ifndef __ENV_I2C_H
#define __ENV_I2C_H

#include "n32l40x.h"

#define I2C2_SCL_Pin_X     GPIO_PIN_2
#define I2C2_SDA_Pin_X     GPIO_PIN_3     

#define I2C2_GPIOX         GPIOA

#define I2C_GenerateSTART(I2CX, CMD)                    I2C_GenerateStart(I2CX, CMD)
#define I2C_Send7bitAddress(I2CX, ADDR, I2C_DIRECTION)  I2C_SendAddr7bit( I2CX, ADDR, I2C_DIRECTION )
#define I2C_GetFlagStatus(I2CX, FLAG)                   I2C_GetFlag(I2CX, FLAG)

#define I2C2_Slave_1_ADDR  (0x44 <<1)
#define I2C2_Slave_2_ADDR  0x44

typedef enum
{
    /* 软件复位命令 */

    SOFT_RESET_CMD = 0x30A2,	
    /*
    单次测量模式
    命名格式：Repeatability_CS_CMD
    CS： Clock stretching
    */
    HIGH_ENABLED_CMD    = 0x2C06,
    MEDIUM_ENABLED_CMD  = 0x2C0D,
    LOW_ENABLED_CMD     = 0x2C10,
    HIGH_DISABLED_CMD   = 0x2400,
    MEDIUM_DISABLED_CMD = 0x240B,
    LOW_DISABLED_CMD    = 0x2416,

    /*
    周期测量模式
    命名格式：Repeatability_MPS_CMD
    MPS：measurement per second
    */
    HIGH_0_5_CMD   = 0x2032,
    MEDIUM_0_5_CMD = 0x2024,
    LOW_0_5_CMD    = 0x202F,
    HIGH_1_CMD     = 0x2130,
    MEDIUM_1_CMD   = 0x2126,
    LOW_1_CMD      = 0x212D,
    HIGH_2_CMD     = 0x2236,
    MEDIUM_2_CMD   = 0x2220,
    LOW_2_CMD      = 0x222B,
    HIGH_4_CMD     = 0x2334,
    MEDIUM_4_CMD   = 0x2322,
    LOW_4_CMD      = 0x2329,
    HIGH_10_CMD    = 0x2737,
    MEDIUM_10_CMD  = 0x2721,
    LOW_10_CMD     = 0x272A,
	/* 周期测量模式读取数据命令 */
	READOUT_FOR_PERIODIC_MODE = 0xE000,
} SHT30_CMD;
/*
-----------------------------------
©著作权归作者所有：来自51CTO博客作者MCUlover666的原创作品，请联系作者获取转载授权，否则将追究法律责任
【STM32Cube_15】使用硬件I2C读取温湿度传感器数据（SHT30）
https://blog.51cto.com/u_13640625/3032434
*/

void I2C2_Configuration(void);
void SHT30_Write(u8 msb, u8 lsb);
int SHT30_ReadTempAndHumi(u8 *data);

uint8_t SHT30_Dat_To_Float(uint8_t* const dat, uint8_t* temperature, uint8_t* humidity);
uint8_t CheckCrc8(uint8_t* const message, uint8_t initial_value);

#endif /* __ENV_I2C_H */
