#include "flash.h"
#include "esp8266.h"

/**
 * @brief 写入flash信息，一共16位，高8位是马达速度，后八位是设定的土壤湿度
 * 
 * @param mortor_level 
 * @param s_hu_setting 
 */
void Flash_Write_Option(uint8_t mortor_level, uint8_t s_hu_setting)
{
    uint32_t flash_write_data;

    flash_write_data = (uint32_t) ( (mortor_level<<8) | s_hu_setting );

    /* Configures the Internal High Speed oscillator */
    if(FLASH_HSICLOCK_DISABLE == FLASH_ClockInit())
    {
        printf("HSI oscillator not yet ready\r\n");
        while(1);
    }
    FLASH_Unlock();

    if (FLASH_COMPL != FLASH_EraseOnePage(FLASH_WRITE_START_ADDR))
    {
        while(1)
        {
            printf("Flash EraseOnePage Error. Please Deal With This Error Promptly\r\n");
        }
    }

    if (FLASH_COMPL != FLASH_ProgramWord(FLASH_WRITE_START_ADDR, flash_write_data))
    {
        while(1)
        {
            printf("Flash ProgramWord Error. Please Deal With This Error Promptly\r\n");
        }
    }
    /* Locks the FLASH Program Erase Controller */
    FLASH_Lock();

    if (flash_write_data != (*(__IO uint32_t*)(FLASH_WRITE_START_ADDR)))
    {
        printf("Flash Program Test Failed\r\n");
    }
    printf( "\r\nData Has been Writen\r\n" );
    // printf( "\r\nData is %x", (* (__IO uint32_t*)(FLASH_WRITE_START_ADDR) ) );
}

/**
 * @brief 读出地址数据
 * 
 * @param addr --所读取的地址
 * @return uint32_t --该地址下的数据
 */
uint32_t Flash_Read_Data( uint32_t addr )
{
    uint32_t data;
    data = (*(__IO uint32_t*)(addr));
    return data;
}
